@@ -86,7 +86,9 @@ class GroupType(EntityType):
         :return: A new instance of GroupType.
         """
         uid = uuid.uuid4()
-        if getattr(entity_class, "default_type_uid", None) is not None:
+        if getattr(entity_class, "default_type_uid", None) is not None and isinstance(
+            entity_class.default_type_uid(), uuid.UUID
+        ):
             uid = entity_class.default_type_uid()
 
             if "ID" in kwargs:
