@@ -25,10 +25,8 @@ from typing import Any
 import h5py
 import numpy as np
 
-from ..data.float_data import FloatData
-from ..data.integer_data import IntegerData
-from ..shared import fetch_h5_handle
-from .utils import as_str_if_uuid, str2uuid, str_from_utf8_bytes
+from ..shared import FLOAT_NDV, INTEGER_NDV, fetch_h5_handle
+from ..shared.utils import KEY_MAP, as_str_if_utf8_bytes, as_str_if_uuid, str2uuid
 
 
 class H5Reader:
@@ -36,18 +34,6 @@ class H5Reader:
     Class to read information from a geoh5 file.
     """
 
-    key_map = {
-        "values": "Data",
-        "cells": "Cells",
-        "surveys": "Surveys",
-        "trace": "Trace",
-        "trace_depth": "TraceDepth",
-        "vertices": "Vertices",
-        "octree_cells": "Octree Cells",
-        "property_groups": "PropertyGroups",
-        "color_map": "Color map",
-    }
-
     @classmethod
     def fetch_attributes(
         cls,
@@ -70,16 +56,14 @@ class H5Reader:
         property_groups: :obj:`dict` of data :obj:`uuid.UUID`
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
             attributes: dict = {"entity": {}}
             type_attributes: dict = {"entity_type": {}}
             property_groups: dict = {}
 
             entity_type = cls.format_type_string(entity_type)
-            if "type" in entity_type:
-                entity_type = entity_type.replace("_", " ") + "s"
-                entity = h5file[name]["Types"][entity_type][as_str_if_uuid(uid)]
-            elif entity_type == "Root":
+
+            if entity_type == "Root":
                 entity = h5file[name][entity_type]
             else:
                 entity = h5file[name][entity_type][as_str_if_uuid(uid)]
@@ -87,52 +71,41 @@ class H5Reader:
             for key, value in entity.attrs.items():
                 attributes["entity"][key] = value
 
-            if "Type" not in entity:
-                entity.create_group("Type")
-
-            for key, value in entity["Type"].attrs.items():
-                type_attributes["entity_type"][key] = value
-
-            if "Color map" in entity["Type"].keys():
-                type_attributes["entity_type"]["color_map"] = {}
-                for key, value in entity["Type"]["Color map"].attrs.items():
-                    type_attributes["entity_type"]["color_map"][key] = value
-                type_attributes["entity_type"]["color_map"]["values"] = entity["Type"][
-                    "Color map"
-                ][:]
-
-            if "Value map" in entity["Type"].keys():
-                mapping = cls.fetch_value_map(file, uid)
-                type_attributes["entity_type"]["value_map"] = mapping
-
+            if "Type" in entity:
+                type_attributes["entity_type"] = cls.fetch_type_attributes(
+                    entity["Type"]
+                )
             # Check if the entity has property_group
-            if "PropertyGroups" in entity.keys():
+            if "PropertyGroups" in entity:
                 property_groups = cls.fetch_property_groups(file, uid)
 
-            attributes["entity"]["existing_h5_entity"] = True
+            attributes["entity"]["on_file"] = True
 
         return attributes, type_attributes, property_groups
 
     @classmethod
-    def fetch_cells(cls, file: str | h5py.File, uid: uuid.UUID) -> np.ndarray | None:
+    def fetch_array_attribute(
+        cls, file: str | h5py.File, uid: uuid.UUID, entity_type: str, key: str
+    ) -> np.ndarray | None:
         """
-        Get an object's :obj:`~geoh5py.objects.object_base.ObjectBase.cells`.
+        Get an entity attribute stores as array such as
+        :obj:`~geoh5py.objects.object_base.ObjectBase.cells`.
 
         :param file: :obj:`h5py.File` or name of the target geoh5 file
         :param uid: Unique identifier of the target object.
+        :param entity_type: Group type to fetch entity from.
+        :param key: Field attribute name.
 
         :return cells: :obj:`numpy.ndarray` of :obj:`int`.
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
-            indices = None
-
+            name = list(h5file)[0]
+            label = KEY_MAP.get(key, key)
             try:
-                indices = h5file[name]["Objects"][as_str_if_uuid(uid)]["Cells"][:]
+                values = h5file[name][entity_type][as_str_if_uuid(uid)][label][:]
+                return values
             except KeyError:
-                pass
-
-        return indices
+                return None
 
     @classmethod
     def fetch_children(
@@ -152,17 +125,21 @@ class H5Reader:
             List of dictionaries for the children uid and type
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
-            children = {}
+            name = list(h5file)[0]
+            children: dict = {}
             entity_type = cls.format_type_string(entity_type)
+
+            if entity_type not in h5file[name]:
+                return children
+
             entity = h5file[name][entity_type][as_str_if_uuid(uid)]
 
             for child_type, child_list in entity.items():
-                if child_type in ["Type", "PropertyGroups"]:
+                if child_type in ["Type", "PropertyGroups", "Concatenated Data"]:
                     continue
 
                 if isinstance(child_list, h5py.Group):
-                    for uid_str in child_list.keys():
+                    for uid_str in child_list:
                         children[str2uuid(uid_str)] = child_type.replace(
                             "s", ""
                         ).lower()
@@ -170,106 +147,126 @@ class H5Reader:
         return children
 
     @classmethod
-    def fetch_delimiters(
+    def fetch_concatenated_values(
         cls,
         file: str | h5py.File,
         uid: uuid.UUID,
-    ) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
+        entity_type: str,
+        label: str,
+    ) -> tuple | None:
         """
-        Get the delimiters of a :obj:`~geoh5py.objects.block_model.BlockModel`.
+        Get :obj:`~geoh5py.shared.entity.Entity.children` values of concatenated group.
 
         :param file: :obj:`h5py.File` or name of the target geoh5 file
-        :param uid: Unique identifier of the target entity.
+        :param uid: Unique identifier
+        :param entity_type: Type of entity from
+            'group', 'data', 'object', 'group_type', 'data_type', 'object_type'
+        :param label: Group identifier for the attribute requested.
 
-
-        Returns
-        -------
-        u_delimiters: :obj:`numpy.ndarray` of u_delimiters
-        v_delimiters: :obj:`numpy.ndarray` of v_delimiters
-        z_delimiters: :obj:`numpy.ndarray` of z_delimiters
+        :return children: [{uuid: type}, ... ]
+            List of dictionaries for the children uid and type
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
+            entity_type = cls.format_type_string(entity_type)
+            label = KEY_MAP.get(label, label)
 
             try:
-                u_delimiters = np.r_[
-                    h5file[name]["Objects"][as_str_if_uuid(uid)]["U cell delimiters"]
+                group = h5file[name][entity_type][as_str_if_uuid(uid)][
+                    "Concatenated Data"
                 ]
-            except KeyError:
-                u_delimiters = None
+                if label not in group["Index"]:
+                    return None
 
-            try:
-                v_delimiters = np.r_[
-                    h5file[name]["Objects"][as_str_if_uuid(uid)]["V cell delimiters"]
-                ]
-            except KeyError:
-                v_delimiters = None
+                if label in group["Data"]:
+                    attribute = group["Data"][label][:]
+                else:
+                    attribute = group[label][:]
 
-            try:
-                z_delimiters = np.r_[
-                    h5file[name]["Objects"][as_str_if_uuid(uid)]["Z cell delimiters"]
-                ]
-            except KeyError:
-                z_delimiters = None
-
-        return u_delimiters, v_delimiters, z_delimiters
-
-    @classmethod
-    def fetch_metadata(cls, file: str | h5py.File, uid: uuid.UUID) -> str | dict | None:
-        """
-        Fetch the metadata of an entity.
-        """
-
-        with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
-
-            try:
-                metadata = np.r_[
-                    h5file[name]["Objects"][as_str_if_uuid(uid)]["Metadata"]
-                ]
-                metadata = str_from_utf8_bytes(metadata[0])
+                return attribute, group["Index"][label][:]
 
             except KeyError:
                 return None
 
-        if isinstance(metadata, str):
-            try:
-                metadata = json.loads(metadata)
-            except ValueError:
-                pass
-
-        if isinstance(metadata, dict):
-            for key, val in metadata.items():
-                if isinstance(val, dict):
-                    for sub_key, sub_val in val.items():
-                        metadata[key][sub_key] = str2uuid(sub_val)
-                else:
-                    metadata[key] = str2uuid(val)
-
-        return metadata
-
     @classmethod
-    def fetch_octree_cells(cls, file: str | h5py.File, uid: uuid.UUID) -> np.ndarray:
+    def fetch_concatenated_attributes(
+        cls,
+        file: str | h5py.File,
+        uid: uuid.UUID,
+        entity_type: str,
+        label: str,
+    ) -> list | dict | None:
         """
-        Get :obj:`~geoh5py.objects.octree.Octree`
-        :obj:`~geoh5py.objects.object_base.ObjectBase.cells`.
+        Get 'Attributes', 'Data' or 'Index' from Concatenator group.
 
         :param file: :obj:`h5py.File` or name of the target geoh5 file
-        :param uid: Unique identifier of the target entity.
+        :param uid: Unique identifier
+        :param entity_type: Type of entity from
+            'group', 'data', 'object', 'group_type', 'data_type', 'object_type'
+        :param label: Group identifier for the attribute requested.
 
-        :return octree_cells: :obj:`numpy.ndarray` of :obj:`int`.
+        :return children: [{uuid: type}, ... ]
+            List of dictionaries for the children uid and type
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
+            entity_type = cls.format_type_string(entity_type)
+            label = KEY_MAP.get(label, label)
 
             try:
-                octree_cells = np.r_[
-                    h5file[name]["Objects"][as_str_if_uuid(uid)]["Octree Cells"]
+                group = h5file[name][entity_type][as_str_if_uuid(uid)][
+                    "Concatenated Data"
                 ]
-            except KeyError:
-                octree_cells = None
+                if label == "Attributes":
+                    attribute = group[label][()]
+                    if isinstance(attribute, np.ndarray):
+                        attribute = attribute[0]
 
-        return octree_cells
+                    return json.loads(as_str_if_utf8_bytes(attribute))
+
+                return list(group[label])
+
+            except KeyError:
+                return None
+
+    @classmethod
+    def fetch_metadata(
+        cls,
+        file: str | h5py.File,
+        uid: uuid.UUID,
+        entity_type: str = "Objects",
+        argument: str = "Metadata",
+    ) -> str | dict | None:
+        """
+        Fetch text of dictionary type attributes of an entity.
+
+        :param file: Target h5 file.
+        :param uid: Unique identifier of the target Entity.
+        :param entity_type: Base type of the target Entity.
+        :param argument: Label name of the dictionary.
+        """
+        with fetch_h5_handle(file) as h5file:
+            name = list(h5file)[0]
+
+            try:
+                metadata = np.r_[
+                    h5file[name][entity_type][as_str_if_uuid(uid)][argument]
+                ]
+                metadata = as_str_if_utf8_bytes(metadata[0])
+
+            except KeyError:
+                return None
+
+        metadata = json.loads(metadata)
+
+        for key, val in metadata.items():
+            if isinstance(val, dict):
+                for sub_key, sub_val in val.items():
+                    metadata[key][sub_key] = str2uuid(sub_val)
+            else:
+                metadata[key] = str2uuid(val)
+
+        return metadata
 
     @classmethod
     def fetch_project_attributes(cls, file: str | h5py.File) -> dict[Any, Any]:
@@ -281,10 +278,13 @@ class H5Reader:
         :return attributes: :obj:`dict` of attributes.
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)
+            if len(name) != 1:
+                raise FileNotFoundError
+
             attributes = {}
 
-            for key, value in h5file[name].attrs.items():
+            for key, value in h5file[name[0]].attrs.items():
                 attributes[key] = value
 
         return attributes
@@ -311,13 +311,13 @@ class H5Reader:
             }
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
             property_groups: dict[str, dict[str, str]] = {}
             try:
                 pg_handle = h5file[name]["Objects"][as_str_if_uuid(uid)][
                     "PropertyGroups"
                 ]
-                for pg_uid in pg_handle.keys():
+                for pg_uid in pg_handle:
                     property_groups[pg_uid] = {}
                     for attr, value in pg_handle[pg_uid].attrs.items():
                         property_groups[pg_uid][attr] = value
@@ -325,6 +325,47 @@ class H5Reader:
                 pass
         return property_groups
 
+    @classmethod
+    def fetch_type(
+        cls, file: str | h5py.File, uid: uuid.UUID, entity_type: str
+    ) -> dict:
+        """
+        Fetch a type from the target geoh5.
+
+        :param file: :obj:`h5py.File` or name of the target geoh5 file
+        :param uid: Unique identifier of the target entity
+        :param entity_type: One of 'Data', 'Object' or 'Group'
+        :return property_group_attributes: :obj:`dict` of property groups
+            and respective attributes.
+
+        """
+        with fetch_h5_handle(file) as h5file:
+            name = list(h5file)[0]
+            entity_type = entity_type + " types"
+            type_handle = h5file[name]["Types"][entity_type][as_str_if_uuid(uid)]
+            return cls.fetch_type_attributes(type_handle)
+
+    @classmethod
+    def fetch_type_attributes(cls, type_handle: h5py.Group) -> dict:
+        """
+        Fetch type attributes from a given h5 handle.
+        """
+        type_attributes = {}
+        for key, value in type_handle.attrs.items():
+            type_attributes[key] = value
+
+        if "Color map" in type_handle:
+            type_attributes["color_map"] = {}
+            for key, value in type_handle["Color map"].attrs.items():
+                type_attributes["color_map"][key] = value
+            type_attributes["color_map"]["values"] = type_handle["Color map"][:]
+
+        if "Value map" in type_handle:
+            mapping = cls.fetch_value_map(type_handle)
+            type_attributes["value_map"] = mapping
+
+        return type_attributes
+
     @classmethod
     def fetch_uuids(cls, file: str | h5py.File, entity_type: str) -> list:
         """
@@ -338,37 +379,33 @@ class H5Reader:
             List of uuids
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
             entity_type = cls.format_type_string(entity_type)
             try:
-                uuids = [str2uuid(uid) for uid in h5file[name][entity_type].keys()]
+                uuids = [str2uuid(uid) for uid in h5file[name][entity_type]]
             except KeyError:
                 uuids = []
 
         return uuids
 
     @classmethod
-    def fetch_value_map(cls, file: str | h5py.File, uid: uuid.UUID) -> dict:
+    def fetch_value_map(cls, h5_handle: h5py.Group) -> dict:
         """
         Get data :obj:`~geoh5py.data.data.Data.value_map`
 
-        :param file: :obj:`h5py.File` or name of the target geoh5 file
-        :param uid: Unique identifier of the target entity
+        :param h5_handle: Handle to the target h5 group.
 
         :return value_map: :obj:`dict` of {:obj:`int`: :obj:`str`}
         """
-        with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
-            try:
-                entity = h5file[name]["Data"][as_str_if_uuid(uid)]
-                value_map = entity["Type"]["Value map"][:]
-                mapping = {}
-                for key, value in value_map.tolist():
-                    value = str_from_utf8_bytes(value)
-                    mapping[key] = value
+        try:
+            value_map = h5_handle["Value map"][:]
+            mapping = {}
+            for key, value in value_map.tolist():
+                value = as_str_if_utf8_bytes(value)
+                mapping[key] = value
 
-            except KeyError:
-                mapping = {}
+        except KeyError:
+            mapping = {}
 
         return mapping
 
@@ -386,7 +423,7 @@ class H5Reader:
         :return values: Data file stored as bytes
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
 
             try:
                 bytes_value = h5file[name]["Data"][as_str_if_uuid(uid)][file_name][
@@ -399,7 +436,9 @@ class H5Reader:
         return bytes_value
 
     @classmethod
-    def fetch_values(cls, file: str | h5py.File, uid: uuid.UUID) -> float | None:
+    def fetch_values(
+        cls, file: str | h5py.File, uid: uuid.UUID
+    ) -> np.ndarray | str | float | None:
         """
         Get data :obj:`~geoh5py.data.data.Data.values`
 
@@ -409,17 +448,17 @@ class H5Reader:
         :return values: :obj:`numpy.array` of :obj:`float`
         """
         with fetch_h5_handle(file) as h5file:
-            name = list(h5file.keys())[0]
+            name = list(h5file)[0]
 
             try:
                 values = np.r_[h5file[name]["Data"][as_str_if_uuid(uid)]["Data"]]
                 if isinstance(values[0], (str, bytes)):
-                    values = str_from_utf8_bytes(values[0])
+                    values = as_str_if_utf8_bytes(values[0])
                 else:
                     if values.dtype in [float, "float64", "float32"]:
-                        ind = values == FloatData.ndv()
+                        ind = values == FLOAT_NDV
                     else:
-                        ind = values == IntegerData.ndv()
+                        ind = values == INTEGER_NDV
                         values = values.astype("float64")
                     values[ind] = np.nan
 
@@ -428,54 +467,9 @@ class H5Reader:
 
         return values
 
-    @classmethod
-    def fetch_coordinates(
-        cls, file: str | h5py.File, uid: uuid.UUID, name: str
-    ) -> np.ndarray | None:
-        """
-        Get an object coordinates data.
-
-        :param file: :obj:`h5py.File` or name of the target geoh5 file
-        :param uid: Unique identifier of the target object
-        :param name: Type of coordinates 'vertices', 'trace' or 'surveys'
-
-        :return surveys: :obj:`numpy.ndarray` of [x, y, z] coordinates
-        """
-        with fetch_h5_handle(file) as h5file:
-            root = list(h5file.keys())[0]
-
-            coordinates = None
-            try:
-                coordinates = np.asarray(
-                    h5file[root]["Objects"][as_str_if_uuid(uid)][cls.key_map[name]]
-                )
-            except KeyError:
-                pass
-
-        return coordinates
-
-    @classmethod
-    def fetch_trace_depth(cls, file: str | h5py.File, uid: uuid.UUID) -> np.ndarray:
-        """
-        Get an object :obj:`~geoh5py.objects.drillhole.Drillhole.trace_depth` data
-
-        :param file: :obj:`h5py.File` or name of the target geoh5 file
-        :param uid: Unique identifier of the target object
-
-        :return surveys: :obj:`numpy.ndarray` of [x, y, z] coordinates
-        """
-        with fetch_h5_handle(file) as h5file:
-
-            try:
-                root = list(h5file.keys())[0]
-                trace_depth = h5file[root]["Objects"][as_str_if_uuid(uid)]["TraceDepth"]
-            except KeyError:
-                trace_depth = None
-
-        return trace_depth
-
     @staticmethod
-    def format_type_string(string):
+    def format_type_string(string: str) -> str:
+        """Format names used for types."""
         string = string.capitalize()
         if string in ["Group", "Object"]:
             string += "s"
