@@ -156,9 +156,9 @@ class Grid2D(ObjectBase):
     def dip(self, value):
         if value is not None:
             assert isinstance(value, float), "Dip angle must be a float"
-            self.modified_attributes = "attributes"
             self._centroids = None
             self._dip = value
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def n_cells(self) -> int | None:
@@ -185,13 +185,13 @@ class Grid2D(ObjectBase):
 
             assert len(value) == 3, "Origin must be a list or numpy array of shape (3,)"
 
-            self.modified_attributes = "attributes"
             self._centroids = None
 
             value = np.asarray(
                 tuple(value), dtype=[("x", float), ("y", float), ("z", float)]
             )
             self._origin = value
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def rotation(self) -> float:
@@ -205,10 +205,9 @@ class Grid2D(ObjectBase):
         if value is not None:
             value = np.r_[value]
             assert len(value) == 1, "Rotation angle must be a float of shape (1,)"
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._rotation = value.astype(float)
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def shape(self) -> tuple | None:
@@ -232,10 +231,9 @@ class Grid2D(ObjectBase):
             value = np.r_[value]
             assert len(value) == 1, "u_cell_size must be a float of shape (1,)"
 
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._u_cell_size = value.astype(float)
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def u_count(self) -> int | None:
@@ -248,11 +246,10 @@ class Grid2D(ObjectBase):
     def u_count(self, value):
         if value is not None:
             value = np.r_[value]
-            assert len(value) == 1, "u_count must be an integer of shape (1,)"
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._u_count = int(value)
+            assert len(value) == 1, "u_count must be an integer of shape (1,)"
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def v_cell_size(self) -> float | None:
@@ -266,10 +263,9 @@ class Grid2D(ObjectBase):
         if value is not None:
             value = np.r_[value]
             assert len(value) == 1, "v_cell_size must be a float of shape (1,)"
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._v_cell_size = value.astype(float)
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def v_count(self) -> int | None:
@@ -283,10 +279,9 @@ class Grid2D(ObjectBase):
         if value is not None:
             value = np.r_[value]
             assert len(value) == 1, "v_count must be an integer of shape (1,)"
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._v_count = int(value)
+            self.workspace.update_attribute(self, "attributes")
 
     @property
     def vertical(self) -> bool | None:
@@ -302,7 +297,6 @@ class Grid2D(ObjectBase):
                 0,
                 1,
             ], "vertical must be of type 'bool'"
-            self.modified_attributes = "attributes"
             self._centroids = None
-
             self._vertical = value
+            self.workspace.update_attribute(self, "attributes")
